#!/bin/sh

#####################################################################
# This script will rename the destination disk image, effectively	#
# "archiving" the disk image for posterity. When the next backup 	#
# task runs, a new disk image will be created. You can get really 	#
# fancy here, e.g. rotating one archive a week out to a monthly		#
# or weekly folder. The default settings keep it simple.			#
#																	#
# You don't need to modify this script at all for basic 		 	# 
# "rename with timestamp" functionality.							#
#																	#
###### Setup procedure within CCC:									#
# 1. In the Finder, create a folder on your destination volume that #
#    is dedicated to storing your collection of archived disk images#
# 2. In the Finder, navigate to your startup disk > Library > 		#
#    Application Support > com.bombich.ccc > Scripts				#
# 3. Copy this "rename_dmg.sh" script into the "Scripts" folder		#
# 4. In CCC, choose your source volume, or choose "Choose a folder" #
#    from the Source selector and select the folder that you want  	#
#	 to back up														#
# 5. Choose "New disk image" from the Destination selector and 		#
#    select the folder that you created in the Finder as the 		#
#	 destination. Be sure to choose a sparse disk image format --	#
#	 if you want to convert the disk image to a read-only format,	#
#	 change the configuration option below.							#
# 6. Click the "User Advanced Settings" button at the bottom of the	#
#	 window															#
# 7. In the "After task runs" section, click on the "Choose script"	#
#    button to specify the "rename_dmg.sh" as a postflight script   #
#####################################################################


# If you want to convert the disk image to a read-only format, change 
# this variable's value to "1". If you use this option, choose the 
# sparseimage disk image format.
convert_to_readonly=0

## No edits required below this line ##

echo "Running $0"
echo "1: $1" >> /dev/stderr  # Source path
echo "2: $2" >> /dev/stderr  # Mounted disk image destination path
echo "3: $3" >> /dev/stderr  # Exit status
echo "4: $4" >> /dev/stderr  # Disk image file path
dmgFile="$4"

if [ "$3" != "0" ]; then
	echo "The backup task exited with a non-zero exit status, skipping the archive postflight task"
	exit 0
fi

# First we need to unmount the disk image. Starting in CCC 4, the disk image is still mounted
# when the postflight script runs to give the user an opportunity to make changes to the
# contents of the mounted volume. We don't need to do anything like that here, we just
# need to unmount it and rename it.
sleep 5 # Especially if this is an APFS disk image, other processes may be "touching" the helper partitions
root_dev=`hdiutil attach "$4" | head -n 1 | awk '{print $1}'`
hdiutil eject "$2"
hdiutil eject "$root_dev"

# parentDir is the folder you chose to save the disk image in, e.g. a "daily" backups folder
parentDir=`dirname "$dmgFile"`
oldName=`basename "$dmgFile"`

if [ "$convert_to_readonly" = "1" ]; then
	oldName=`basename -s .sparseimage "$dmgFile"`
	tmpName="${oldName}".dmg
	echo "Calling hdiutil convert $dmgFile -ov -format UDZO -o $parentDir/$tmpName"
	hdiutil convert "$dmgFile" -ov -format UDZO -o "$parentDir/$tmpName"
	if [ "$?" = "0" ]; then
		# Delete the read-write copy
		rm "$dmgFile"
		oldName="$tmpName"
	else
		oldName=`basename "$dmgFile"`
	fi
fi

newName=`date '+%Y-%m-%d-%H-%M'`_"$oldName"

# rename the disk image
if [ -d "$parentDir" ]; then
	cd "$parentDir"
	echo "mv \"$oldName\" \"$newName\"" >> /dev/stderr
	mv "$oldName" "$newName"

	### Fancy stuff ###
	## Prune disk images older than 14 days
	## Consider that this removes *any* file from the parent folder that holds your disk images,
	## it is not restricted to just these disk image files. If you implement this pruning
	## procedure, you should take care to store only disk images in the folder that stores your
	## CCC destination disk image

	# find . -mtime +14 -exec rm '{}' \;
fi

exit 0
